//========================================================================
// SCIDYNE Corporation   649 School Street   Pembroke, MA 02359-3649 USA
// Tel: (781) 293-3059   Fax: (781) 293-4034   URL: http://www.scidyne.com 
//========================================================================
// NOTICE:
//    This work is licensed under a Creative Commons Attribution-ShareAlike 3.0 Unported License. 
//    http://creativecommons.org/licenses/by-sa/3.0/us/
//
//========================================================================
// Project  :  XMEM+                                        
// File     :  xmem_plus_ex1.ino                                             
// Revision :  1.00                                               
// Date     :  12-03-15 
// Author:  :  Mark Durgin
// Target   :  Arduino Mega 2560, Arduino Mega ADK
// Compiler :  Arduino IDE 1.6.6                                                
//========================================================================
// Description and usage: 
//  This sketch outlines the basic steps required when only the 32KB of
//  bank-0 (default) will be used for the heap, i.e., no bank switching. 
//  Two areas in the XMEM+ SRAM will be allocated and their resulting addresses
//  displayed on the IDE Serial Monitor. No special libraries are required.
//  
//  It's worth noting that the sizes requested far exceed what a standard
//  Arduino MEGA2560 would be capable if not for the XMEM+.
//========================================================================
// Issues, Concerns and "THINGS TO DO":
//
//========================================================================
// History:
//  12-03-15:
//    - Sketch first written
//========================================================================


/*******************************************************************************************
 * Important program parameters
 *******************************************************************************************/
 #define XMEM_START   ( (void *) 0x2200 )   // Start of XMEM+ External SRAM memory space
 #define XMEM_END     ( (void *) 0xA1FF )   // End of XMEM+ External SRAM memory space

 /*******************************************************************************************
  * Global variables
  *******************************************************************************************/ 
  char buf[80];                        // A display buffer for using sprintf 
  
  // Create some pointers for the purpose of this example
  // Note: It's good practice to use descriptive names including the associated bank number
  unsigned char *   combuf_bank0;      // Will hold the start address of some communication buffer (uses 8-bit chars)
           int  *   big_array_bank0;   // Will hold the start address of some big integer array (uses 16-bit integer)

/********************************************************************************************
 * Program initialization
 ********************************************************************************************/
void setup()
{

  // Setup for using External Memeory and Expansion Bus
  XMCRB  =  0x00;             // All of all PORTC pins act as upper address lines, A[15:8], disable Bus keeper function 
  XMCRA  =  0x80;             // Set SRE bit, Enable xmem+, No wait states. If Wait-states are needed change this value accordingly

  // Relocate the heap to XMEM+
  __malloc_heap_end   =  static_cast <char *> ( XMEM_END );     // Relocate end of heap area
  __malloc_heap_start =  static_cast <char *> ( XMEM_START );   // Relaocate start of heap area
              

  // Initialize the pointers.
  // Note: The use of sizeof() isn't required but it does make determining the number of bytes to request much easier, especially when allocating
  // more complex, multi-byte, data types such as structures.
  combuf_bank0    = (unsigned char *) malloc( 12000 * sizeof( unsigned char ));  // Request a 12,000 byte area within the heap, returns the starting address
  big_array_bank0 = (int *)           malloc(  8000 * sizeof( int ));            // Request an 8,000 int (16,000 byte) area within the heap, returns the starting address

  // Setup communications to IDE serial monitor
  Serial.begin(9600);  

  // Display the addresses on the serial monitor
  sprintf(buf,"__malloc_heap_start    : %04X", __malloc_heap_start);
  Serial.println(buf);
  
  sprintf(buf,"__malloc_heap_end      : %04X", __malloc_heap_end);
  Serial.println(buf);
  
  sprintf(buf,"combuf_bank0 start     : %04X", combuf_bank0);
  Serial.println(buf);
  
  sprintf(buf,"big_array_bank0 start  : %04X", big_array_bank0);
  Serial.println(buf);
  Serial.println(); Serial.println();

  // ...
  // Any other user initialization code goes here
  // ...

}


//************************************************************************
// This is the main loop
//************************************************************************
void loop()
{
  
  // ...
  // In an actual program, the code to use the areas allocated
  // for the global *combuf_bank0 and *big_array_bank0
  // might go here or in other functions
  // ...


  // ...
  // Any other user code goes here
  // ...
  
}
