//========================================================================
// SCIDYNE Corporation   649 School Street   Pembroke, MA 02359-3649 USA
// Tel: (781) 293-3059   Fax: (781) 293-4034   URL: www.scidyne.com
//========================================================================
// NOTICE:
//    This work is licensed under a Creative Commons Attribution-ShareAlike 3.0 Unported License. 
//    http://creativecommons.org/licenses/by-sa/3.0/us/
//
//========================================================================
// Project  :  XMEM+ Tests                                     
// File     :  xmem_plus_ex3.ino                                             
// Revision :  1.00                                               
// Date     :  7-31-15 
// Author:  :  Mark Durgin
// Target   :  Arduino Mega 2560, Arduino Mega ADK
// Compiler :  Arduino IDE 1.6.6                                                 
//========================================================================
// Description and usage: 
//   This program uses the expansion bus of the XMEM+ by connecting to
//   an 82C55 Peripheral Interface chip.
//
//   For simplicity, this test keeps all the heap and other system parameters
//   in the Arduino internal memory.
//========================================================================
// Issues, Concerns and "THINGS TO DO":
//   - The slower access times of the 82C55A requires wait-states to be added, XMCRA  =  0x8F;
//   - The 82C55A has an active high reset input, opposite of what the XMEM+ RESOUT signal provides
//     so a digital output as is used as the chip reset for this test.
//========================================================================
// History:
//  07-31-15  R0.00   - Project Begins
//========================================================================

// Setup Pointers to access the 82C55 Chip
unsigned char *  PRTA = (unsigned char *) 0xA200;    // 82C55 PORTA Register
unsigned char *  PRTB = (unsigned char *) 0xA201;    // 82C55 PORTB Register
unsigned char *  PRTC = (unsigned char *) 0xA202;    // 82C55 PORTC Register
unsigned char *  CTRL = (unsigned char *) 0xA203;    // 82C55 Control Register

// Global variables created in .bss area
char buf[80];     // A display buffer for using sprintf 


//************************************************************************
void setup()
{
        // Activate external memory
        XMCRB  =  0x00;        // All of all PORTC pins act as upper address lines, A[15:8], disable Bus keeper function 
	      XMCRA  =  0xD4;        // Set SRE bit. Divide space in to two sectors; Lower sector 0x2200-0x9FFF, Upper sector 0xA000-0xFFFF 
                               //   Apply 1 wait state on Upper segment because the 82C55A is too slow for XMEM+ Parallel Expansion bus
        
        // Setup 82C55
        pinMode(7, OUTPUT);      // Digital #7 to be used as reset for 82C55A
        digitalWrite(7, HIGH);   // Reset 82C55A
        delay(25);               // wait ...
        digitalWrite(7, LOW);    // Remove 82C55A reset
        
        *CTRL = 0x8A;            // All 82C55A ports are to be Outputs
 
        Serial.begin(9600);      // Start serial communications for IDE debugging monitor
     
        sprintf(buf,"Testing Expansion Bus");
        Serial.println(buf);
}


//************************************************************************
void loop()
{
   unsigned char i, b;
 
    for ( i = 0; i < 255; i++ )  // Make a binary counter 
    {
       *PRTA = i;     // Write value to PORTA
       b = *PRTA;     // Read back the PORT
      
      // Compare and display error if mis-match is found
      if ( i != b )
      {
        sprintf(buf,"Fail: %02X != %02X", i, b);  // Display the mis-match error
        Serial.println(buf);
      }
    }
}

